#include <math.h>
#include "picosystem.hpp"

using namespace picosystem;

uint32_t view = 0;
uint32_t view_count = 4;

void init() {
}

void update(uint32_t tick) {
  if(pressed(RIGHT)) {view = view == (view_count - 1) ? 0 : view + 1;}
  if(pressed( LEFT)) {view = view == 0 ? (view_count - 1) : view - 1;}
}

void title(std::string t) {
  pen(15, 15, 15);
  frect(0, 0, 120, 11);
  pen(0, 0, 0);
  text(t + " (" + str(view + 1) + "/" + str(view_count) + ")", 2, 2);
}

extern color_t image_data[1440];
buffer_t douglas = {.w = 32, .h = 45, .data = image_data};

void draw(uint32_t tick) {
  pen(0, 0, 0);
  clear();

  switch(view) {
    case 2: {
      title("Word wrap");

      uint32_t wrap = (sin(time() / 500.0f) * 36.0f) + 80;

      font();

      // define a long message to word wrap
      std::string message = "\
\\penAAAFWe are just an advanced breed of \\penFFFFmonkeys\\penAAAF on a minor \
planet of a \\penFFFFvery average star\\penAAAF. But we can understand the \
Universe. That makes us something very special.\
\n\
\n\t\\penFFFF- Stephen Hawking";

      // measure how large the text will be and draw boundary on screen
      int32_t w, h;
      measure(message, w, h, wrap);
      pen(4, 4, 4, 4);
      frect(0, 28, w + 4, h + 4);

      // draw wrapped text
      pen(8, 8, 8);
      text(message, 2, 30, wrap);

      // draw wrap width marker
      pen(15, 15, 15);
      text("Wrap here ", wrap - 54, 15);
      vline(wrap + 2, 15, 8);
      line(wrap + 2, 23, wrap + 2 - 2, 21);
      line(wrap + 2, 23, wrap + 2 + 2, 21);
      pen(0, 8, 0);
      vline(wrap + 2, 27, 92);
    } break;

    case 1: {
      title("Colour codes");

      font(-1, 12);

      pen(15, 15, 15);
      std::string message = "\
\\penffff~680nm is \\penf00fred\\penffff\
\n~480nm is \\pen00ffblue\\penffff\
\nYou're on my wavelength\
\nAnd I quite like your hue\
\n\
\n\\pen0fff- Tanya Ha (@Ha_Tanya)";

      pen(8, 8, 8);
      text(message, 2, 32);
    }break;

    case 3: {
      font(-1, 12);

      std::string message = "\
\"The fact that we live at the bottom of a deep gravity well, on the surface of \
a gas covered planet going around a nuclear fireball 90 million miles away and \
think this to be normal is obviously some indication of how skewed our \
perspective tends to be.\"";

      // box width and height
      uint32_t bx = 30, by = 0;
      uint32_t bw = 90, bh = 120;

      int32_t w, h, p = 20;
      measure(message, w, h, bw - 10);
      h += p * 2;
      int32_t overflow = h - bh;
      int32_t scroll = (sin(time() / 2000.0f) * overflow / 2.0f) + (overflow / 2.0f);
      scroll -= p;

      alpha(4);
      blit(&douglas, 0, 0, 32, 45, -10, 20 - (scroll / 3), 32 * 3, 45 * 3);

      alpha();
      pen(12, 12, 12);
      clip(bx, by, bw, bh);
      text(message, bx + 5, by + 5 - scroll, bw - 10);
      clip();

      title("Scroll and clip");
    }break;

    case 0: {
      font(-1, 12);

      std::string message = "\
Lo, brave adventurer! Choose a tasty treat:\
\n\
\n   A) Ripe and tasty \\spr001\
\n   B) Delicious \\spr000\
\n   C) This firm \\spr004\
\n   D) Very spooky \\spr013\
\n\
\nMake your choice...";

      pen(15, 15, 15);
      text(message, 2, 16, 116);

      title("Inline icons");
    }break;
  }
}

color_t image_data[1440] = {
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x11f1, 0x44f4,
  0x44f4, 0x33f3, 0x22f2, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x11f1, 0x66f6, 0x88f8, 0x66f6,
  0x66f6, 0x88f8, 0x66f6, 0x44f4, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x22f2, 0x77f7, 0x88f8, 0x77f7, 0x88f8,
  0x77f7, 0x77f7, 0x88f8, 0x88f8, 0x66f6, 0x22f2, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x11f1, 0x44f4, 0x55f5, 0x55f5, 0x66f6, 0x55f5, 0x55f5, 0x44f4,
  0x66f6, 0x77f7, 0x99f9, 0x99f9, 0x99f9, 0x88f8, 0x55f5, 0x11f1,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x33f3,
  0xaafa, 0xbbfb, 0x88f8, 0x55f5, 0x44f4, 0x00f0, 0x00f0, 0x33f3,
  0x88f8, 0xaafa, 0xaafa, 0xaafa, 0xaafa, 0x88f8, 0x77f7, 0x55f5,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x66f6, 0x77f7, 0x88f8,
  0xbbfb, 0xccfc, 0x99f9, 0x77f7, 0x66f6, 0x00f0, 0x00f0, 0x77f7,
  0xaafa, 0xbbfb, 0xccfc, 0xbbfb, 0xaafa, 0x88f8, 0x77f7, 0x55f5,
  0x22f2, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x33f3, 0xccfc, 0x99f9, 0x99f9,
  0xccfc, 0xeefe, 0xccfc, 0xaafa, 0xaafa, 0x55f5, 0x66f6, 0xbbfb,
  0xddfd, 0xddfd, 0xddfd, 0xbbfb, 0x99f9, 0x88f8, 0x66f6, 0x55f5,
  0x33f3, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x55f5, 0xaafa, 0x77f7, 0xaafa,
  0xddfd, 0xffff, 0xffff, 0xffff, 0xeefe, 0xddfd, 0xddfd, 0xeefe,
  0xeefe, 0xeefe, 0xddfd, 0xbbfb, 0xaafa, 0x88f8, 0x77f7, 0x55f5,
  0x33f3, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x55f5, 0x88f8, 0x88f8, 0xccfc,
  0xeefe, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff,
  0xffff, 0xffff, 0xddfd, 0xccfc, 0xbbfb, 0x99f9, 0x77f7, 0x55f5,
  0x33f3, 0x11f1, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x77f7, 0x99f9, 0xaafa, 0xddfd,
  0xeefe, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff,
  0xffff, 0xffff, 0xeefe, 0xddfd, 0xbbfb, 0x99f9, 0x77f7, 0x55f5,
  0x33f3, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x77f7, 0x88f8, 0xaafa, 0xccfc,
  0xeefe, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff,
  0xffff, 0xffff, 0xffff, 0xddfd, 0xbbfb, 0x99f9, 0x77f7, 0x55f5,
  0x33f3, 0x11f1, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x88f8, 0x66f6, 0x99f9, 0xccfc,
  0xeefe, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff,
  0xffff, 0xffff, 0xffff, 0xddfd, 0xbbfb, 0x99f9, 0x77f7, 0x55f5,
  0x33f3, 0x11f1, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x77f7, 0x66f6, 0x88f8, 0xccfc,
  0xddfd, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff,
  0xffff, 0xffff, 0xffff, 0xddfd, 0xbbfb, 0x99f9, 0x77f7, 0x55f5,
  0x33f3, 0x11f1, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x56f6, 0x66f6, 0x99f9, 0xccfc,
  0xddfd, 0xeefe, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff,
  0xffff, 0xffff, 0xffff, 0xddfd, 0xbbfb, 0x99f9, 0x77f7, 0x55f5,
  0x22f2, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x44f4, 0x33f3, 0xaafa, 0xffff,
  0xeefe, 0xeefe, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff,
  0xffff, 0xffff, 0xeefe, 0xddfd, 0xbbfb, 0x99f9, 0x77f7, 0x55f5,
  0x22f2, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x11f1, 0x22f2, 0x77f7, 0x99f9,
  0xddfd, 0xeefe, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff,
  0xffff, 0xffff, 0xffff, 0xeefe, 0xbbfb, 0x88f8, 0x66f6, 0x44f4,
  0x22f2, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x11f1, 0x00f0, 0x00f0, 0x00f0,
  0x11f1, 0x11f1, 0x11f1, 0x66f6, 0xffff, 0xffff, 0xeefe, 0xeefe,
  0xddfd, 0xaafa, 0x66f6, 0x44f4, 0x33f3, 0x44f4, 0x11f1, 0x00f0,
  0x22f2, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x11f1, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x88f8, 0xffff, 0xffff,
  0x88f8, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x11f1, 0x33f3, 0x44f4, 0x88f8, 0x00f0, 0x00f0, 0xeefe, 0xffff,
  0xaafa, 0x66f6, 0x44f4, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x11f1,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x44f4, 0x88f8, 0x22f2,
  0x66f6, 0xccfc, 0xffff, 0xffff, 0xddfd, 0x11f1, 0xbbfb, 0xeefe,
  0xddfd, 0xddfd, 0xbbfb, 0x77f7, 0x66f6, 0x55f5, 0x33f3, 0x22f2,
  0x11f1, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x11f1, 0xccfc, 0x88f8,
  0x66f6, 0xaafa, 0xddfd, 0xffff, 0x99f9, 0x33f3, 0xccfc, 0xddfd,
  0xccfc, 0xddfd, 0x99f9, 0x99f9, 0x88f8, 0x55f5, 0x44f4, 0x22f2,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x88f8, 0xabfb,
  0x99f9, 0xbbfb, 0xeefe, 0xeefe, 0x55f5, 0x33f3, 0xddfd, 0xddfd,
  0xaafa, 0xccfc, 0xbbfb, 0x88f8, 0x66f6, 0x44f4, 0x44f4, 0x22f2,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x22f2, 0x99f9,
  0xccfc, 0xccfc, 0xeefe, 0xbbfb, 0x33f3, 0x44f4, 0xddfd, 0xeefe,
  0xaafa, 0xbbfb, 0xccfc, 0xaafa, 0x77f7, 0x66f6, 0x44f4, 0x11f1,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x77f7,
  0xbbfb, 0xccfc, 0xddfd, 0x88f8, 0x33f3, 0x88f8, 0xddfd, 0xeefe,
  0xddfd, 0xaafa, 0xbbfb, 0xbbfb, 0x88f8, 0x66f6, 0x33f3, 0x11f1,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x44f4,
  0x99f9, 0xccfc, 0x99f9, 0x11f1, 0x55f5, 0xaafa, 0xeefe, 0xffff,
  0xddfd, 0xccfc, 0x88f8, 0x99f9, 0x88f8, 0x66f6, 0x33f3, 0x11f1,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x22f2,
  0x88f8, 0xccfc, 0x77f7, 0x00f0, 0x00f0, 0xaafa, 0xffff, 0xffff,
  0xaafa, 0x77f7, 0xaafa, 0x66f6, 0x77f7, 0x66f6, 0x33f3, 0x11f1,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x66f6, 0x99f9, 0x66f6, 0x00f0, 0x00f0, 0x22f2, 0xaafa, 0x88f8,
  0x66f6, 0x88f8, 0xbbfb, 0x77f7, 0x44f4, 0x55f5, 0x33f3, 0x11f1,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x11f1,
  0x33f3, 0x55f5, 0x88f8, 0x33f3, 0x00f0, 0x00f0, 0x00f0, 0x11f1,
  0xaafa, 0xccfc, 0x99f9, 0x88f8, 0x55f5, 0x44f4, 0x33f3, 0x11f1,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x33f3,
  0x44f4, 0x00f0, 0x00f0, 0x44f4, 0x22f2, 0x11f1, 0x44f4, 0x88f8,
  0xbbfb, 0xaafa, 0x88f8, 0x66f6, 0x55f5, 0x44f4, 0x22f2, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x66f6, 0x44f4, 0x00f0, 0x11f1, 0x44f4, 0x66f6, 0x99f9, 0x88f8,
  0x88f8, 0x77f7, 0x44f4, 0x33f3, 0x55f5, 0x44f4, 0x22f2, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x44f4, 0x66f6, 0x33f3, 0x22f2, 0x33f3, 0x44f4, 0x22f2, 0x44f4,
  0x66f6, 0x77f7, 0x77f7, 0x66f6, 0x55f5, 0x44f4, 0x11f1, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x11f1, 0x55f5, 0x66f6, 0x55f5, 0x44f4, 0x55f5, 0x66f6, 0x44f4,
  0x55f5, 0x77f7, 0x88f8, 0x77f7, 0x55f5, 0x33f3, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x33f3, 0x77f7, 0xbbfb, 0xccfc, 0xbbfb, 0xccfc, 0xbbfb,
  0xbbfb, 0xaafa, 0x99f9, 0x77f7, 0x55f5, 0x22f2, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x33f3, 0xaafa, 0xbbfb, 0xccfc, 0xeefe, 0xeefe,
  0xccfc, 0xbbfb, 0xaafa, 0x77f7, 0x33f3, 0x00f0, 0x11f1, 0x11f1,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x22f2, 0x99f9, 0xccfc, 0xddfd, 0xddfd,
  0xccfc, 0xbbfb, 0x88f8, 0x11f1, 0x00f0, 0x11f1, 0x33f3, 0x22f2,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x44f4, 0x77f7, 0x77f7,
  0x55f5, 0x44f4, 0x00f0, 0x00f0, 0x22f2, 0x66f6, 0x44f4, 0x11f1,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x55f5, 0x88f8, 0x66f6, 0x55f5, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x11f1, 0x77f7, 0xaafa, 0x88f8, 0x77f7, 0x33f3, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x33f3, 0x77f7, 0x99f9, 0xaafa, 0x99f9, 0x88f8, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x22f2, 0x22f2, 0x00f0, 0x00f0, 0x00f0, 0x55f5,
  0x77f7, 0xaafa, 0xccfc, 0xbbfb, 0xaafa, 0x44f4, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x66f6, 0xbbfb, 0xaafa, 0x66f6, 0x55f5, 0x99f9,
  0xccfc, 0xccfc, 0xddfd, 0xccfc, 0x99f9, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x77f7, 0xeefe, 0xeefe, 0xccfc, 0xccfc, 0xddfd,
  0xddfd, 0xddfd, 0xccfc, 0xddfd, 0x33f3, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x66f6, 0xffff, 0xffff, 0xffff, 0xffff, 0xeefe,
  0xeefe, 0xeefe, 0xeefe, 0x99f9, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
  0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0, 0x00f0,
};